@@ -1,4 +1,6 @@
 /* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
+#include <string.h>
+
 #include "hippo-canvas-internal.h"
 #include "hippo-canvas-item.h"
 #include "hippo-canvas-container.h"
@@ -14,6 +16,7 @@ enum {
     BUTTON_PRESS_EVENT,
     BUTTON_RELEASE_EVENT,
     MOTION_NOTIFY_EVENT,
+    SCROLL_EVENT,
     KEY_PRESS_EVENT,
     ACTIVATED,
     TOOLTIP_CHANGED,
@@ -153,6 +156,21 @@ hippo_canvas_item_base_init(void *klass)
                           g_signal_accumulator_true_handled, NULL,
                           hippo_canvas_marshal_BOOLEAN__BOXED,
                           G_TYPE_BOOLEAN, 1, HIPPO_TYPE_EVENT);
+
+        /**
+         * HippoCanvasItem::scroll-event
+         *
+         * Signal emitted when the mouse wheel or other mechanism requests scrolling.
+         */                
+        signals[SCROLL_EVENT] =
+            g_signal_new ("scroll-event",
+                          HIPPO_TYPE_CANVAS_ITEM,
+                          G_SIGNAL_RUN_LAST,
+                          G_STRUCT_OFFSET(HippoCanvasItemIface, scroll_event),
+                          g_signal_accumulator_true_handled, NULL,
+                          hippo_canvas_marshal_BOOLEAN__BOXED,
+                          G_TYPE_BOOLEAN, 1, HIPPO_TYPE_EVENT);
+        
         /**
          * HippoCanvasItem::key-press-event
          *
@@ -231,6 +249,14 @@ hippo_canvas_item_destroy(HippoCanvasItem *canvas_item)
     g_object_run_dispose(G_OBJECT(canvas_item));
 }
 
+HippoCanvasContext *
+hippo_canvas_item_get_context(HippoCanvasItem *canvas_item)
+{
+    g_return_val_if_fail(HIPPO_IS_CANVAS_ITEM(canvas_item), NULL);
+
+    return HIPPO_CANVAS_ITEM_GET_IFACE(canvas_item)->get_context(canvas_item);
+}
+
 void
 hippo_canvas_item_set_context(HippoCanvasItem    *canvas_item,
                               HippoCanvasContext *context)
@@ -320,9 +346,22 @@ hippo_canvas_item_get_tooltip(HippoCanvasItem *canvas_item,
                               int              y,
                               HippoRectangle  *for_area)
 {
+    char *tip;
+    
     g_return_val_if_fail(HIPPO_IS_CANVAS_ITEM(canvas_item), NULL);
 
-    return HIPPO_CANVAS_ITEM_GET_IFACE(canvas_item)->get_tooltip(canvas_item, x, y, for_area);
+    tip = HIPPO_CANVAS_ITEM_GET_IFACE(canvas_item)->get_tooltip(canvas_item, x, y, for_area);
+
+    /* Supporting "" as a synonym for NULL is a convenience for pygtk, which
+     * doesn't have support for NULL-returns from interface methods
+     * at the moment.
+     */
+    if (tip != NULL && strcmp(tip, "") == 0) {
+        g_free(tip);
+        tip = NULL;
+    }
+
+    return tip;
 }
 
 HippoCanvasPointer
@@ -458,6 +497,26 @@ hippo_canvas_item_emit_motion_notify_event (HippoCanvasItem  *canvas_item,
     return result;
 }
 
+gboolean
+hippo_canvas_item_emit_scroll_event (HippoCanvasItem     *canvas_item,
+                                     int                  x,
+                                     int                  y,
+                                     HippoScrollDirection direction)
+{
+    HippoEvent event;
+    gboolean result;
+    
+    g_return_val_if_fail(HIPPO_IS_CANVAS_ITEM(canvas_item), FALSE);
+
+    event.type = HIPPO_EVENT_SCROLL;
+    event.x = x;
+    event.y = y;    
+    event.u.scroll.direction = direction;
+    
+    result = hippo_canvas_item_process_event(canvas_item, &event, 0, 0);
+
+    return result;
+}
 
 gboolean
 hippo_canvas_item_emit_key_press_event (HippoCanvasItem  *canvas_item,
@@ -574,6 +633,9 @@ hippo_canvas_item_process_event(HippoCanvasItem *canvas_item,
     case HIPPO_EVENT_KEY_PRESS:
         g_signal_emit(canvas_item, signals[KEY_PRESS_EVENT], 0, &translated, &handled);
         break;
+    case HIPPO_EVENT_SCROLL:
+        g_signal_emit(canvas_item, signals[SCROLL_EVENT], 0, &translated, &handled);
+        break;
         /* don't add a default, you'll break the compiler warnings */
     }
 
